let TRANSLATE_SUCCESS = chrome.i18n.getMessage("translate_success"), TRANSLATE_FAIL = chrome.i18n.getMessage("translate_fail"), TRANSLATEBASE_50001 = chrome.i18n.getMessage("translate_without_login"), TRANSLATEBASE_50005 = chrome.i18n.getMessage("translate_without_subscribe");

class TranslationTask {
    constructor(e, t) {
        this.id = e, this.params = t, this.abortController = new AbortController(), 
        this.resolve = null, this.reject = null, this.promise = new Promise((e, t) => {
            this.resolve = e, this.reject = t;
        });
    }
}

class BaseTranslator {
    constructor() {
        this.activeTasks = new Map(), this.pendingTasks = [], this.maxConcurrent = 5, 
        this.timeout = 3e4;
    }
    addTask(e, t) {
        e = new TranslationTask(e, t);
        return this.pendingTasks.push(e), this.processQueue(), {
            promise: e.promise
        };
    }
    cancelTask(t) {
        var e = this.pendingTasks.findIndex(e => e.id === t);
        return -1 !== e ? (this.pendingTasks.splice(e, 1)[0].reject({
            code: 500,
            data: null,
            message: "The user aborted a request."
        }), !0) : !!this.activeTasks.has(t) && (this.activeTasks.get(t).abortController.abort(), 
        !0);
    }
    processQueue() {
        for (;this.activeTasks.size < this.maxConcurrent && 0 < this.pendingTasks.length; ) {
            var e = this.pendingTasks.shift();
            this.activeTasks.set(e.id, e), this.executeTask(e);
        }
    }
    async executeTask(e) {}
}

class MicrosoftTranslator extends BaseTranslator {
    constructor() {
        super(), this.authCode = null, this.authPromise = null, this.authExpireTime = null;
    }
    async executeTask(t) {
        var {
            textArr: e,
            fromLangCode: a,
            toLangCode: r
        } = t.params;
        let s = null;
        try {
            await this.getAuthCode(), s = setTimeout(() => t.abortController.abort(), this.timeout);
            var o = await fetch(`https://api.cognitive.microsofttranslator.com/translate?&from=${a}&to=${r}&api-version=3.0`, {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    authorization: "Bearer " + this.authCode
                },
                body: JSON.stringify(e.map(e => ({
                    Text: e
                }))),
                signal: t.abortController.signal
            }), n = (clearTimeout(s), await o.json());
            if (!Array.isArray(n) || n.length !== e.length) throw new Error(n.error?.message);
            t.resolve({
                code: 200,
                data: n.map(e => e.translations[0].text),
                message: TRANSLATE_SUCCESS
            });
        } catch (e) {
            s && clearTimeout(s), t.reject({
                code: 500,
                data: null,
                message: e.message || TRANSLATE_FAIL
            });
        } finally {
            this.activeTasks.delete(t.id), this.processQueue();
        }
    }
    getAuthCode() {
        return this.authExpireTime && this.authExpireTime > Date.now() ? Promise.resolve(this.authCode) : (this.authPromise || (this.authPromise = new Promise((t, a) => {
            let e = new AbortController(), r = setTimeout(() => e.abort(), 3e4);
            fetch("https://edge.microsoft.com/translate/auth", {
                method: "GET",
                signal: e.signal
            }).then(e => e.text()).then(e => {
                clearTimeout(r), this.authCode = e, this.authExpireTime = Date.now() + 6e5, 
                t(e);
            }).catch(e => {
                clearTimeout(r), this.authCode = null, this.authExpireTime = null, 
                a(e);
            }).finally(() => {
                this.authPromise = null;
            });
        })), this.authPromise);
    }
}

class GoogleTranslator extends BaseTranslator {
    constructor() {
        super();
    }
    async executeTask(t) {
        var {
            textArr: e,
            fromLangCode: a,
            toLangCode: r
        } = t.params;
        let s = null;
        try {
            s = setTimeout(() => t.abortController.abort(), this.timeout);
            var o = await fetch(`https://translate.googleapis.com/translate_a/t?client=gtx&dt=t&sl=${a}&tl=` + r, {
                method: "POST",
                headers: {
                    "Content-Type": "application/x-www-form-urlencoded"
                },
                body: e.map(e => "q=" + encodeURI(e)).join("&"),
                signal: t.abortController.signal
            }), n = (clearTimeout(s), await o.json());
            if (!Array.isArray(n) || n.length !== e.length) throw new Error(n.message);
            t.resolve({
                code: 200,
                data: n.map(e => Array.isArray(e) ? e[0] : e),
                message: TRANSLATE_SUCCESS
            });
        } catch (e) {
            s && clearTimeout(s), t.reject({
                code: 500,
                data: null,
                message: e.message || TRANSLATE_FAIL
            });
        } finally {
            this.activeTasks.delete(t.id), this.processQueue();
        }
    }
}

class TranslatebaseTranslator extends BaseTranslator {
    constructor(e) {
        super(), this.baseUrl = e;
    }
    async executeTask(t) {
        try {
            var a = 1 === t.params.textArr.length;
            let e = null;
            if (!(e = a ? await this.singleTranslate(t) : await this.translate(t)).success) throw "50001" === String(e.errorCode) && t.reject({
                code: 50001,
                data: null,
                message: TRANSLATEBASE_50001
            }), "50005" === String(e.errorCode) && t.reject({
                code: 50005,
                data: null,
                message: TRANSLATEBASE_50005
            }), new Error(e.message || TRANSLATE_FAIL);
            var r = a ? [ e.translationText ] : JSON.parse(e.translationText);
            if (r.length !== t.params.textArr.length) throw new Error(TRANSLATE_FAIL);
            t.resolve({
                code: 200,
                data: r,
                message: TRANSLATE_SUCCESS
            });
        } catch (e) {
            t.reject({
                code: 500,
                data: null,
                message: e.message || TRANSLATE_FAIL
            });
        } finally {
            this.activeTasks.delete(t.id), this.processQueue();
        }
    }
    translate(s) {
        let {
            service: o,
            textArr: n,
            fromLangId: i,
            toLangId: l
        } = s.params;
        return new Promise((t, a) => {
            var e = new FormData();
            e.append("text", JSON.stringify(n)), e.append("dataAiID", o.id), e.append("fromLanguageID", i), 
            e.append("toLanguageID", l), e.append("url", s.params.translateUrl), 
            e.append("requestSource", "plugin");
            let r = setTimeout(() => s.abortController.abort(), this.timeout);
            fetch(this.baseUrl + "/Ai/translationText", {
                method: "POST",
                body: e,
                signal: s.abortController.signal
            }).then(e => e.json()).then(e => {
                clearTimeout(r), t(e);
            }).catch(e => {
                clearTimeout(r), a(e);
            });
        });
    }
    singleTranslate(s) {
        let {
            service: o,
            textArr: n,
            fromLangId: i,
            toLangId: l
        } = s.params;
        return new Promise((t, a) => {
            var e = new FormData();
            e.append("text", n[0]), e.append("dataAiID", o.id), e.append("fromLanguageID", i), 
            e.append("toLanguageID", l), e.append("url", s.params.translateUrl), 
            e.append("requestSource", "plugin");
            let r = setTimeout(() => s.abortController.abort(), this.timeout);
            fetch(this.baseUrl + "/Ai/translationTextNew", {
                method: "POST",
                body: e,
                signal: s.abortController.signal
            }).then(e => e.json()).then(e => {
                clearTimeout(r), t(e);
            }).catch(e => {
                clearTimeout(r), a(e);
            });
        });
    }
}

class ClaudeAITranslator extends BaseTranslator {
    constructor() {
        super(), this.maxConcurrent = 4;
    }
    async executeTask(t) {
        var {
            service: e,
            textArr: a,
            toLangCode: r
        } = t.params;
        let s = null;
        try {
            var o = {
                model: "claude-3-5-sonnet-20241022",
                max_tokens: 2048,
                temperature: 0,
                messages: [ {
                    role: "user",
                    content: [ "你是一个专业的，正宗的机器翻译引擎。", "您将获得一个数组格式的文本，将数组中的每一项翻译为" + r + "，如果有空字符串就返回空字符串，将翻译结果保持数组格式放于标签<translation></translation>中，以下是输入内容:", JSON.stringify(a), "请直接返回<translation>标签包裹的内容，不要包含任何附加信息。" ].join("\n")
                } ]
            }, n = (s = setTimeout(() => t.abortController.abort(), this.timeout), 
            await fetch("https://api.anthropic.com/v1/messages", {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    "x-api-key": e.config.apiKey,
                    "anthropic-version": "2023-06-01",
                    "anthropic-dangerous-direct-browser-access": !0
                },
                body: JSON.stringify(o),
                signal: t.abortController.signal
            })), i = (clearTimeout(s), await n.json());
            if (i.error) throw new Error(i.error.message);
            var l = i.content[0].text.match(new RegExp("<translation>(.*?)</translation>", "s")), c = JSON.parse(l[1]);
            if (c.length !== a.length) throw new Error(TRANSLATE_FAIL);
            t.resolve({
                code: 200,
                data: c,
                message: TRANSLATE_SUCCESS
            });
        } catch (e) {
            s && clearTimeout(s), t.reject({
                code: 500,
                data: null,
                message: e.message || TRANSLATE_FAIL
            });
        } finally {
            this.activeTasks.delete(t.id), this.processQueue();
        }
    }
}

class DeepSeekTranslator extends BaseTranslator {
    constructor() {
        super();
    }
    async executeTask(t) {
        var {
            service: e,
            textArr: a,
            toLangCode: r
        } = t.params;
        let s = null;
        try {
            var o = {
                model: "deepseek-chat",
                temperature: 1.1,
                messages: [ {
                    role: "system",
                    content: "你是一个专业的，正宗的机器翻译引擎。"
                }, {
                    role: "user",
                    content: [ "您将获得一个数组格式的文本，将数组中的每一项翻译为" + r + "，如果有空字符串就返回空字符串，将翻译结果保持数组格式放于标签<translation></translation>中，以下是输入内容:", JSON.stringify(a), "请直接返回<translation>标签包裹的内容，不要包含任何附加信息。" ].join("\n")
                } ]
            }, n = (s = setTimeout(() => t.abortController.abort(), this.timeout), 
            await fetch("https://api.deepseek.com/chat/completions", {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    Authorization: "Bearer " + e.config.apiKey
                },
                body: JSON.stringify(o),
                signal: t.abortController.signal
            })), i = (clearTimeout(s), await n.json());
            if (i.error) throw new Error(i.error?.message || TRANSLATE_FAIL);
            var l = i.choices[0].message.content.match(new RegExp("<translation>(.*?)</translation>", "s")), c = JSON.parse(l[1]);
            if (c.length !== a.length) throw new Error(TRANSLATE_FAIL);
            t.resolve({
                code: 200,
                data: c,
                message: TRANSLATE_SUCCESS
            });
        } catch (e) {
            s && clearTimeout(s), t.reject({
                code: 500,
                data: null,
                message: e.message || TRANSLATE_FAIL
            });
        } finally {
            this.activeTasks.delete(t.id), this.processQueue();
        }
    }
}

class DeepLTranslator extends BaseTranslator {
    constructor() {
        super();
    }
    async executeTask(t) {
        let {
            service: e,
            textArr: a,
            toLangCode: r
        } = t.params, s = null;
        try {
            "no" === (r = "zh-TW" === (r = "zh-CN" === r ? "ZH-HANS" : r) ? "ZH-HANT" : r) && (r = "nb"), 
            s = setTimeout(() => t.abortController.abort(), this.timeout);
            var o = await fetch("https://api.deepl.com/v2/translate", {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    Authorization: "DeepL-Auth-Key " + e.config.apiKey
                },
                body: JSON.stringify({
                    text: a,
                    target_lang: r
                }),
                signal: t.abortController.signal
            }), n = (clearTimeout(s), await o.json());
            if (n.error) throw new Error(n.error.message);
            t.resolve({
                code: 200,
                data: n.translations.map(e => e.text),
                message: TRANSLATE_SUCCESS
            });
        } catch (e) {
            s && clearTimeout(s), t.reject({
                code: 500,
                data: null,
                message: e.message || TRANSLATE_FAIL
            });
        } finally {
            this.activeTasks.delete(t.id), this.processQueue();
        }
    }
}

class DoubaoTranslator extends BaseTranslator {
    constructor() {
        super();
    }
    async executeTask(t) {
        var {
            service: e,
            textArr: a,
            toLangCode: r
        } = t.params;
        let s = null;
        try {
            var o = {
                model: "ep-20250318200321-kfxdz",
                temperature: .2,
                max_tokens: 4096,
                messages: [ {
                    role: "system",
                    content: "你是一个专业的，正宗的机器翻译引擎。"
                }, {
                    role: "user",
                    content: [ "您将获得一个数组格式的文本，将数组中的每一项翻译为" + r + "，如果有空字符串就返回空字符串，将翻译结果保持数组格式放于标签<translation></translation>中，以下是输入内容:", JSON.stringify(a), "请直接返回<translation>标签包裹的内容，不要包含任何附加信息。" ].join("\n")
                } ]
            }, n = (s = setTimeout(() => t.abortController.abort(), this.timeout), 
            await fetch("https://ark.cn-beijing.volces.com/api/v3/chat/completions", {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    Authorization: "Bearer " + e.config.apiKey
                },
                body: JSON.stringify(o),
                signal: t.abortController.signal
            })), i = (clearTimeout(s), await n.json());
            if (i.error) throw new Error(i.error.message);
            var l = i.choices[0].message.content.match(new RegExp("<translation>(.*?)</translation>", "s")), c = JSON.parse(l[1]);
            if (c.length !== a.length) throw new Error(TRANSLATE_FAIL);
            t.resolve({
                code: 200,
                data: c,
                message: TRANSLATE_SUCCESS
            });
        } catch (e) {
            s && clearTimeout(s), t.reject({
                code: 500,
                data: null,
                message: e.message || TRANSLATE_FAIL
            });
        } finally {
            this.activeTasks.delete(t.id), this.processQueue();
        }
    }
}

let translator = {
    baseUrl: "",
    providers: new Map(),
    taskProviderMap: new Map(),
    addTask: async function(a, r) {
        var s = r.service;
        try {
            let e = s.id, t = (s.config && "TranslateBase" === s.config.provider && (e = "TranslateBase"), 
            this.providers.get(e));
            t || (t = this.createProvider(e, s), this.providers.set(e, t));
            var o = t.addTask(a, r), n = (this.taskProviderMap.set(a, t), await o.promise);
            return n;
        } catch (e) {
            return {
                code: e.code || 500,
                data: null,
                message: e.message || TRANSLATE_FAIL
            };
        } finally {
            this.taskProviderMap.delete(a);
        }
    },
    cancelTask: function(e) {
        e.forEach(e => this.taskProviderMap.get(e)?.cancelTask(e));
    },
    createProvider: function(e, t) {
        if ("microsoft_4191bebefb" === e) return new MicrosoftTranslator();
        if ("google_a42e45fceabe7" === e) return new GoogleTranslator();
        if ("TranslateBase" === e) return new TranslatebaseTranslator(this.baseUrl);
        if ("CustomKey" === t.config?.provider) {
            if (!t.config.apiKey) throw e = chrome.i18n.getMessage("translate_engine_" + t.id) || t.name, 
            e = chrome.i18n.getMessage("options_api_key_required_with_name", [ e ]), 
            new Error(e);
            switch (t.id) {
              case "24051611331075367592":
                return new ClaudeAITranslator();

              case "24082310264087513038":
                return new DeepSeekTranslator();

              case "24093015132425024428":
                return new DeepLTranslator();

              case "24111114370015012781":
                return new DoubaoTranslator();
            }
        }
        throw new Error("Unsupported translation service: " + t.name);
    }
};

export default translator;